/* Copycat game for the LPC1114
 * 4 LED's and 4 buttons are connected to the LPC1114 as well as a buzzer
 * Each round of the game requires the user to copy a randomly generated
 * pattern of lights and beeps by pressing the button beside each light
 * in the correct sequence.  A particular tone is associated with each 
 * light.  Each round lengthens light/tone pattern by one.  The player
 * loses when they fail to copy correctly or if there is a timeout.
 *
 * IO List:
 * CT32B1_MAT0 (PIN 10) = Buzzer
 * PIO1_2 (PIN 11) = Blue LED
 * PIO1_3 (PIN 12) = Green LED
 * PIO1_4 (PIN 13) = Red LED
 * PIO1_5 (PIN 14) = White LED
 * 
 * PIO0_8 (PIN 1)  = Blue button
 * PIO0_9 (PIN 2)  = Green button
 * PIO0_10 (PIN 3) = Red button
 * PIO0_11 (PIN 4) = White button
 */

#include "lpc111x.h"
volatile unsigned msCounter=0; // declare a global millisecond counter for timing
							   // purposes
unsigned prbs(void);
				 
void initSysTick()
{
	
	// The systick timer is driven by a 48MHz clock
	// Divide this down to achieve a 1ms timebase
	// Divisor = 48MHz/1000Hz
	// Reload value = 48000-1
	// enable systick and its interrupts
	SYST_CSR |=(BIT0+BIT1+BIT2); 
	SYST_RVR=48000-1; // generate 1 millisecond time base
	SYST_CVR=5;
	enable_interrupts();
}
void SysTick(void)
{
	msCounter++;
	prbs(); // cycle the random number generator
}
void configPins()
{
	SYSAHBCLKCTRL |= BIT6 + BIT16; // Turn on clock for GPIO and IOCON 
	
	// Buzzer pin configuration
	// Make pin 10 behave as a PWM output CT32B1_MAT0 (with pullup)
	IOCON_R_PIO1_1 |= BIT1 + BIT0; 
	
	// LED pin cofiguration
	// Make port 1 bit 2 behave as a generic IO port
	IOCON_R_PIO1_2 |= BIT0;
	// Make port 1 bit 3 behave as a generic IO port
	IOCON_SWDIO_PIO1_3 |= BIT0;
	// Port 1 bit 4 behaves as a generic IO port by default
	// Port 1 bit 5 behaves as a generic IO port by default
	// Make LED pins behave as outputs
	GPIO1DIR |= (BIT2+BIT3+BIT4+BIT5);
	GPIO1DATA=0;
	// Button pin configuration
	// PIO0_8 and PIO0_9 are inputs by default with pull-ups enabled
	// Make Port 0 bit 10 behave as a generic I/O port (with pullup)
	IOCON_SWCLK_PIO0_10  |= BIT0; 
	// Make Port 0 bit 11 behave as a generic I/O port 
	IOCON_R_PIO0_11  |= BIT0; 
	
	
}
void initPWM()
{
	// will use counter/timer CT32B1
	// Turn on CT32B1
	SYSAHBCLKCTRL |= BIT10;
	// Use match register 3 as period register because its output
	// is not pinned out.  A value of 48000000 produces a frequency of 1Hz
	// so, to generate a 30kHz pwm signal, set MR3 = 48000000/30000 = 1600	
	TMR32B1MR3 = 1600; 
	TMR32B1MR0 = 1600; // Zero output to begin with
	TMR32B1MCR = BIT10; // Reset TC on match with MR3
	TMR32B1TC = 0 ; // Zero the counter to begin with
	TMR32B1PWMC = BIT0; // Enable PWM on channel 0
	TMR32B1TCR = 1; // Enable the timer	
	
}
void delay(unsigned len)
{
	unsigned ms;
	
	ms = msCounter;
	while ((msCounter - ms) < len);
}
void White(unsigned Duration)
{
	GPIO1DATA |= BIT5;
	setFrequency(174);
	delay(Duration);
	GPIO1DATA &= ~BIT5;
	setFrequency(0);
}
void Red(unsigned Duration)
{
	GPIO1DATA |= BIT4;
	setFrequency(392);
	delay(Duration);
	GPIO1DATA &= ~BIT4;
	setFrequency(0);
}
void Green(unsigned Duration)
{
	GPIO1DATA |= BIT3;
	setFrequency(440);
	delay(Duration);
	GPIO1DATA &= ~BIT3;
	setFrequency(0);
}
void Blue(unsigned Duration)
{
	GPIO1DATA |= BIT2;
	setFrequency(349);
	delay(Duration);
	GPIO1DATA &= ~BIT2;
	setFrequency(0);
}

void All(unsigned Duration)
{
	GPIO1DATA |= (BIT2+BIT3+BIT4+BIT5);
	setFrequency(261);
	delay(Duration);
	GPIO1DATA &= ~(BIT2+BIT3+BIT4+BIT5);
	setFrequency(0);

}
void setFrequency(int Frequency)
{
	int Divisor;
	TMR32B1TCR = 0; // Enable the timer	
	Divisor = 48000000/Frequency;
	TMR32B1MR3 = Divisor; 
	TMR32B1MR0 = Divisor >> 1; 
	TMR32B1TC = 0 ; // Zero the counter to begin with
	TMR32B1TCR = 1; // Enable the timer	
}
void setVolume(int Volume)
{	
	// sets the volume to a value in the range 0 to 100%
	int MatchValue;
	MatchValue = TMR32B1MR3;
	MatchValue = ((MatchValue * Volume) / 100)>>1;  // a duty of 50% produces most noise
	TMR32B1MR0 = MatchValue;
}
void Intro()
{
	Red(500);
	Green(500);
	Blue(500);
	White(500);
	All(500);
	delay(500);
	while(pollButtons()==0)
	{
		All(50);
		delay(50);
	}
}
int pollButtons()
{
	if ((GPIO0DATA & BIT8)==0)
		return 1;		
	else if ((GPIO0DATA & BIT9)==0)
		return 2;
	else if ((GPIO0DATA & BIT10)==0)
		return 3;
	else if ((GPIO0DATA & BIT11)==0)
		return 4;
	else
		return 0;
}
unsigned prbs()
{
	// This is an unverified 31 bit PRBS generator
	// It should be maximum length but this has not been verified 
	static unsigned long shift_register=0xa5a5a5a5;
	unsigned long new_bit=0;
	static int busy=0; // need to prevent re-entrancy here
	if (!busy)
	{
		busy=1;
		new_bit= ((shift_register & (1<<27))>>27) ^ ((shift_register & (1<<30))>>30);
		new_bit= ~new_bit;
		new_bit = new_bit & 1;
		shift_register=shift_register << 1;
		shift_register=shift_register | (new_bit);
		busy=0;
	}
	return shift_register & 0x7ffffff; // return 31 LSB's 
}	
void playSequence(int *Seq,int len)
{
	int i;
	for (i=0;i<len;i++)
	{
		switch (Seq[i]) {
			case 0: Blue(500); break;
			case 1: Green(500); break;
			case 2: Red(500); break;
			case 3: White(500); break;
		}
		delay(500);
	}
	// Signal the user that it is their turn
	All(50);
	delay(50);
	All(50);
	delay(50);
}
int getUserSequence(int *Seq,int len)
{
	int Count=0;
	int Btn;
	while (Count < len)
	{		
		while(pollButtons()==0); // wait for press
		Btn = pollButtons()-1;
		if (Btn==0)
			Blue(100);
		if (Btn==1)
			Green(100);
		if (Btn==2)		
			Red(100);
		if (Btn==3)	
			White(100);		
		if (Btn != Seq[Count])
			return -1;
		while(pollButtons()!=0); // wait for release
		delay(20);
		Count++;
	}
	return 0; // correct sequence entered
}
int main()
{	
	int Sequence[8];  // up to 8 tones in a sequence
	int NextNote;
	int Round=0;
	initSysTick();
	configPins();
	initPWM();
	Intro();
	while(1) 
	{
		
		while(Round < 8)
		{
			NextNote=prbs() & 3;
			if (Round > 0)
			{
				// don't allow same note twice in a row
				while (NextNote==Sequence[Round-1])
					NextNote=prbs() & 3;
			}
			Sequence[Round]=NextNote;
			Round++;
			playSequence(Sequence,Round);
			if (getUserSequence(Sequence,Round) < 0 )
			{
				// User failed to copy correctly
				while(1)
				{
					Red(100);
					White(100);
				}
			
			}
			delay(500);
		}
		// Success!
		while(1)
		{
				White(50);
				Red(50);
				Green(50);
				Blue(50);
		}
	}    
}



